/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2007 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/


#ifndef __TERRALIB_INTERNAL_TEFUNCTIONCALLTHREADJOB_H
#define __TERRALIB_INTERNAL_TEFUNCTIONCALLTHREADJOB_H

#include "TeThreadJob.h"
#include "TeDefines.h"

    
/*! \brief This job calls a C function passing a user supplied
 * parameters pointer.
 * \author Emiliano F. Castejon
 * \ingroup MultProgToolsGroup
 */
class TL_DLL TeFunctionCallThreadJob : public TeThreadJob
{
  public :
    /** \brief Type definition for a thread job entry function pointer.
     *  \param parsPtr The function user parameters pointer.
     */
    typedef void (*FunctPtrT)( void* parsPtr );      
    
    /*! A pointer to the function that will be called when this job
     * stats
     */ 
    FunctPtrT jobFunc_;

    /*! A pointer to the function paramters that will be passed when this job
     * stats
     */      
    void* jobFuncParsPtr_;
    
    TeFunctionCallThreadJob();
    
    ~TeFunctionCallThreadJob();
    
    //Overloaded.
    TeThreadJob* clone() const;
    
    // Overloaded
    void executeJobCode();
};
      
  
#endif

