#include "TePDIParaSegStrategyFactoryParams.hpp"

TePDIParaSegStrategyFactoryParams::TePDIParaSegStrategyFactoryParams()
{
}


TePDIParaSegStrategyFactoryParams::~TePDIParaSegStrategyFactoryParams()
{
}

std::string TePDIParaSegStrategyFactoryParams::decName() const
{
  return std::string();
}

