#include "TePDIParaSegStrategyParams.hpp"

TePDIParaSegStrategyParams::TePDIParaSegStrategyParams()
: globalMutexPtr_( 0 ), segmentsIdsCounterPtr_( 0 ), 
  blockMergingEnabled_( false )
{
}


TePDIParaSegStrategyParams::~TePDIParaSegStrategyParams()
{
}

